@@ -34,6 +34,7 @@
 #include <fcitx/candidate.h>
 #include <fcitx-config/xdg.h>
 #include <fcitx/module/pinyin/fcitx-pinyin.h>
+#include <fcitx/module/freedesktop-notify/fcitx-freedesktop-notify.h>
 
 #include "cloudpinyin.h"
 #include "fetch.h"
@@ -88,14 +89,15 @@ static CloudPinyinCache* CloudPinyinAddToCache(FcitxCloudPinyin* cloudpinyin, co
 static INPUT_RETURN_VALUE CloudPinyinGetCandWord(void* arg, FcitxCandidateWord* candWord);
 static void _CloudPinyinAddCandidateWord(FcitxCloudPinyin* cloudpinyin, const char* pinyin);
 static void CloudPinyinFillCandidateWord(FcitxCloudPinyin* cloudpinyin, const char* pinyin);
-static boolean LoadCloudPinyinConfig(FcitxCloudPinyinConfig* fs);
-static void SaveCloudPinyinConfig(FcitxCloudPinyinConfig* fs);
+static boolean CloudPinyinConfigLoad(FcitxCloudPinyinConfig* fs);
+static void CloudPinyinConfigSave(FcitxCloudPinyinConfig* fs);
 static char *GetCurrentString(FcitxCloudPinyin* cloudpinyin,
                               char **ascii_part);
 static void CloudPinyinHookForNewRequest(void* arg);
 static CURL* CloudPinyinGetFreeCurlHandle(FcitxCloudPinyin* cloudpinyin);
 static void CloudPinyinReleaseCurlHandle(FcitxCloudPinyin* cloudpinyin,
                                          CURL* curl);
+static INPUT_RETURN_VALUE CloudPinyinToggle(void* arg);
 
 CloudPinyinEngine engine[4] =
 {
@@ -172,7 +174,7 @@ void* CloudPinyinCreate(FcitxInstance* instance)
     int pipe1[2];
     int pipe2[2];
 
-    if (!LoadCloudPinyinConfig(&cloudpinyin->config))
+    if (!CloudPinyinConfigLoad(&cloudpinyin->config))
     {
         free(cloudpinyin);
         return NULL;
@@ -227,6 +229,13 @@ void* CloudPinyinCreate(FcitxInstance* instance)
     FcitxInstanceRegisterInputUnFocusHook(instance, hook);
     FcitxInstanceRegisterTriggerOnHook(instance, hook);
 
+    FcitxHotkeyHook hkhook;
+    hkhook.arg = cloudpinyin;
+    hkhook.hotkey = cloudpinyin->config.hkToggle.hotkey;
+    hkhook.hotkeyhandle = CloudPinyinToggle;
+
+    FcitxInstanceRegisterHotkeyFilter(instance, hkhook);
+
     pthread_create(&cloudpinyin->pid, NULL, FetchThread, fetch);
 
     CloudPinyinRequestKey(cloudpinyin);
@@ -272,7 +281,7 @@ void CloudPinyinAddCandidateWord(void* arg)
     FcitxIM* im = FcitxInstanceGetCurrentIM(cloudpinyin->owner);
     FcitxInputState* input = FcitxInstanceGetInputState(cloudpinyin->owner);
 
-    if (cloudpinyin->initialized == false)
+    if (!cloudpinyin->initialized || !cloudpinyin->config.bEnabled)
         return;
 
     /* check whether the current im is pinyin */
@@ -424,7 +433,7 @@ void CloudPinyinReloadConfig(void* arg)
 {
     FcitxCloudPinyin* cloudpinyin = (FcitxCloudPinyin*) arg;
     CloudPinyinSource previousSource = cloudpinyin->config.source;
-    LoadCloudPinyinConfig(&cloudpinyin->config);
+    CloudPinyinConfigLoad(&cloudpinyin->config);
     if (previousSource != cloudpinyin->config.source)
     {
         cloudpinyin->initialized = false;
@@ -589,6 +598,29 @@ CloudPinyinCache* CloudPinyinAddToCache(FcitxCloudPinyin* cloudpinyin, const cha
     return cacheEntry;
 }
 
+INPUT_RETURN_VALUE CloudPinyinToggle(void* arg)
+{
+    FcitxCloudPinyin* cloudpinyin = (FcitxCloudPinyin*) arg;
+    FcitxInstance* instance = cloudpinyin->owner;
+    FcitxIM* im = FcitxInstanceGetCurrentIM(cloudpinyin->owner);
+
+    if (CHECK_VALID_IM) {
+        cloudpinyin->config.bEnabled = !cloudpinyin->config.bEnabled;
+
+        FcitxFreeDesktopNotifyShowAddonTip(
+            instance, "fcitx-punc-toggle",
+            "fcitx",
+            _("Cloud Pinyin"),
+            cloudpinyin->config.bEnabled ? _("Cloud Pinyin is Enabled.") :
+                                    _("Cloud Pinyin is Disabled."));
+        CloudPinyinConfigSave(&cloudpinyin->config);
+        // TODO: add a notification here
+
+        return IRV_DO_NOTHING;
+    }
+    return IRV_TO_PROCESS;
+}
+
 void _CloudPinyinAddCandidateWord(FcitxCloudPinyin* cloudpinyin, const char* pinyin)
 {
     CloudPinyinCache* cacheEntry = CloudPinyinCacheLookup(cloudpinyin, pinyin);
@@ -683,12 +715,14 @@ void CloudPinyinFillCandidateWord(FcitxCloudPinyin* cloudpinyin,
         for (i = 0;i < size &&
                  (cand = FcitxCandidateWordGetByTotalIndex(candList, i));i++) {
             if (strcmp(cand->strWord, cacheEntry->str) == 0) {
+                uint64_t ts = cloudCand->timestamp;
+                uint64_t curTs = CloudGetTimeStamp();
                 FcitxCandidateWordRemove(candList, candWord);
                 /* if cloud word is not on the first page.. impossible */
                 if (cloudidx < pagesize) {
                     /* if the duplication before cloud word */
                     if (i < cloudidx) {
-                        if (CloudGetTimeStamp() - cloudCand->timestamp
+                        if (curTs - ts
                             > LOADING_TIME_QUICK_THRESHOLD) {
                             FcitxCandidateWordInsertPlaceHolder(candList, cloudidx);
                             FcitxCandidateWord* placeHolder = FcitxCandidateWordGetByTotalIndex(candList, cloudidx);
@@ -699,8 +733,7 @@ void CloudPinyinFillCandidateWord(FcitxCloudPinyin* cloudpinyin,
                         if (i >= pagesize) {
                             FcitxCandidateWordMove(candList, i - 1, cloudidx);
                         } else {
-                            if (CloudGetTimeStamp() - cloudCand->timestamp
-                                > LOADING_TIME_QUICK_THRESHOLD) {
+                            if (curTs - ts > LOADING_TIME_QUICK_THRESHOLD) {
                                 FcitxCandidateWordInsertPlaceHolder(candList, cloudidx);
                                 FcitxCandidateWord* placeHolder = FcitxCandidateWordGetByTotalIndex(candList, cloudidx);
                                 if (placeHolder && placeHolder->strWord == NULL)
@@ -780,7 +813,7 @@ INPUT_RETURN_VALUE CloudPinyinGetCandWord(void* arg, FcitxCandidateWord* candWor
  *
  * @param Bool is reload or not
  **/
-boolean LoadCloudPinyinConfig(FcitxCloudPinyinConfig* fs)
+boolean CloudPinyinConfigLoad(FcitxCloudPinyinConfig* fs)
 {
     FcitxConfigFileDesc *configDesc = GetCloudPinyinConfigDesc();
     if (configDesc == NULL)
@@ -791,7 +824,7 @@ boolean LoadCloudPinyinConfig(FcitxCloudPinyinConfig* fs)
     if (!fp)
     {
         if (errno == ENOENT)
-            SaveCloudPinyinConfig(fs);
+            CloudPinyinConfigSave(fs);
     }
     FcitxConfigFile *cfile = FcitxConfigParseConfigFileFp(fp, configDesc);
     FcitxCloudPinyinConfigConfigBind(fs, cfile, configDesc);
@@ -808,7 +841,7 @@ boolean LoadCloudPinyinConfig(FcitxCloudPinyinConfig* fs)
  *
  * @return void
  **/
-void SaveCloudPinyinConfig(FcitxCloudPinyinConfig* fs)
+void CloudPinyinConfigSave(FcitxCloudPinyinConfig* fs)
 {
     FcitxConfigFileDesc *configDesc = GetCloudPinyinConfigDesc();
     FILE *fp = FcitxXDGGetFileUserWithPrefix("conf", "fcitx-cloudpinyin.config", "w", NULL);
@@ -887,22 +920,6 @@ char *GetCurrentString(FcitxCloudPinyin* cloudpinyin, char **ascii_part)
     }
 }
 
-void SogouParseKey(FcitxCloudPinyin* cloudpinyin, CurlQueue* queue)
-{
-    char* str = fcitx_utils_trim(queue->str);
-    const char* ime_patch_key = "ime_patch_key = \"";
-    size_t len = strlen(str);
-    if (len == SOGOU_KEY_LENGTH + strlen(ime_patch_key) + 1
-        && strncmp(str, ime_patch_key, strlen(ime_patch_key)) == 0
-        && str[len - 1] == '\"') {
-        sscanf(str,"ime_patch_key = \"%s\"", cloudpinyin->key);
-        cloudpinyin->initialized = true;
-        cloudpinyin->key[SOGOU_KEY_LENGTH] = '\0';
-    }
-
-    free(str);
-}
-
 void CloudPinyinHookForNewRequest(void* arg)
 {
     FcitxCloudPinyin* cloudpinyin = (FcitxCloudPinyin*) arg;
